import os
import sys
import traceback
from signal import SIGCHLD
from optparse import OptionParser

import gobject

import dbus
import dbus.service
import dbus.mainloop.glib

from rainbow import util, inject
from rainbow.util.linux.clone import clone, CLONE_NEWNS

def log(msg, *args):
    if len(args):
        print msg % args
    else:
        print msg

class Rainbow(dbus.service.Object):
    """The Rainbow security service."""
    SERVICE_NAME = 'org.laptop.security.Rainbow'
    INTERFACE_NAME = 'org.laptop.security.Rainbow'

    _foreground_xid = None
    _id_map = {}

    def __init__(self, bus_or_name):
        dbus.service.Object.__init__(self, bus_or_name, '/')

    @dbus.service.method(INTERFACE_NAME,
                         in_signature='s',
                         out_signature='s')
    def echo(self, msg):
        return msg

    @dbus.service.method('org.laptop.security.DBus',
                         in_signature='si',
                         out_signature='b')
    def CheckCanOwn(self, service_name, xid):
        """Check the the given xid is allowed to bind to the bus with the given
        name.
        """

        # XXX: This should be checking that the thing on the other side is really
        # the session bus. <NPK>

        return service_name.startswith('org.laptop.Container%s'%xid)

    @dbus.service.method(INTERFACE_NAME,
                         in_signature='s')
    def ChangeActivity(self, activity_id):
        """Sets CPU throttling on the foreground and background activities as needed."""
        pass

    @dbus.service.method(INTERFACE_NAME,
                         in_signature='sa{ss}asss',
                         async_callbacks=('success_cont', 'error_cont'),
                         utf8_strings=True)
    def CreateActivity(self, log_path, env, argv, bundle_path, bundle_id,
                       success_cont, error_cont):
        # Reap zombies
        try:
            while os.waitpid(-1, os.WNOHANG) != (0,0):
                pass
        except OSError:
            pass

        try:
            child_pid = clone(CLONE_NEWNS | SIGCHLD)
            if not child_pid:
                # XXX: I _do not_ yet understand which fds I need to close when.
                # For example - should the D-Bus FD needs to be open to send
                # error messages? If so, when does it get closed?
                try:
                    log_fd = os.open(log_path, os.O_WRONLY | os.O_SYNC)
                    os.dup2(log_fd, 1)
                    os.dup2(log_fd, 2)
                    # FIXME: here come the magic numbers and strings
                    ret = inject.run(log, env, argv, env['SUGAR_BUNDLE_PATH'],
                            500, 500, bundle_path, bundle_id, '/home/olpc/isolation/1')
                except Exception, e:
                    util.trace()
                    error_cont(e)
                finally:
                    os._exit(255)

            success_cont()

        except Exception, e:
            util.trace()
            error_cont(e)

def run():
    """Start the Rainbow DBus service."""
    parser = OptionParser(version='0.1')
    parser.add_option('-f', '--config-file', default='/etc/rainbow/rainbow.conf',
                      help='The configuration file for Rainbow.')
    parser.add_option('-d', '--daemon', action='store_true', default=False,
                      help='Really daemonize; e.g. detach from the controlling terminal.')
    opts, args = parser.parse_args()

    if opts.daemon:
        util.daemon(0, 1)
        open('/var/run/rainbow.pid', 'w').write(str(os.getpid()))
        sys.stdout = open('/var/log/rainbow', 'w', 0)
        sys.stderr = sys.stdout

    dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)

    bus = dbus.SystemBus()
    name = dbus.service.BusName(Rainbow.SERVICE_NAME, bus)
    Rainbow(name)

    print 'Service running mainloop.'
    mainloop = gobject.MainLoop()
    mainloop.run()

def main():
    run()

if __name__ == '__main__':
    main()
