import os.path
import shutil

__all__ = ['components', 'make_dirs', 'get_mounted_paths', 'rm']

def components(p):
    """Take a path and return a list of its components:
    e.g. /abc/de or /abc/de/ ---> ['abc', 'de']"""
    path = os.path.realpath(p)
    (head, tail) = os.path.split(path)
    if tail == '':
        (head, tail) = os.path.split(head)
    cpts = []
    while tail != '':
        cpts = [tail] + cpts
        (head, tail) = os.path.split(head)
    return cpts

def make_dirs(pth, uid, gid, mode):
    """A replacement for os.makedirs that sets ownership as it creates
    directories."""
    if os.path.exists(pth):
        return
    root = '/'
    cpts = components(pth)
    for cpt in components(pth):
        root = os.path.join(root, cpt)
        if not os.path.exists(root):
            #print "Made %s for (%d, %d)" % (root, uid, gid)
            # XXX: A picky person would be worried about races here.
            os.mkdir(root, mode)
            os.chown(root, uid, gid)

def get_mounted_paths():
    mounts = [] 
    procfile = open('/proc/mounts')
    for line in procfile.readlines():
        mounts.append(line.split(' ')[1])
    procfile.close()
    return mounts

def rm(path):
    if os.path.isdir(path):
        shutil.rmtree(path)
    else:
        os.unlink(path)
